require_relative 'player'
require_relative 'gameturn'
require_relative 'treasures'

module StudioGame
        class Game

            attr_reader :title
            attr_accessor :players

            def initialize(title)
                @title = title.capitalize
                @players = []
            end

            def add_player(player)
                @players << player
            end

            def high_score(player)
                player.name.ljust(20, '.') + "#{player.score}"
            end

            def sort_players
                @players.sort { |a, b| b.score <=> a.score }
            end

            def load_player(from_file)
                File.readlines(from_file).each do |line|
                    add_player(Player.from_csv(line))
                end
            end

            def save_scores(to_file = "highscores.txt")
                File.open(to_file, "w") do |file|
                    file.puts "#{@title}'s High Scores:"
                    @players.sort { |a, b| b.score <=> a.score }.each do |player|
                        file.puts high_score(player)
                    end
                end
            end

            def play(turns)
                puts "There are #{@players.size} players in #{@title.capitalize}:"
                @players.each do |player|
                    puts player
                end

                treasures = TreasureChest::TREASURES
                puts "\nThere are #{treasures.size} treasures in the game:"
                treasures.each do |t|
                    puts "A #{t.name} is worth #{t.points} points."
                end

                1.upto(turns) do |turn|
                    puts "\nRound #{turn}:"
                    @players.each do |player|
                        GameTurn.take_turn(player)
                    end
                end
            end

            def print_stats
                strong, wimpy = @players.partition { |p| p.strong? }

                puts "\n#{@title}'s Statistics:"
                puts "\n#{strong.size} strong players:"

                strong.each do |p|
                    puts "#{p.name}(#{p.health})"
                end

                puts "\n#{wimpy.size} wimpy players:"
                wimpy.each do |p|
                    puts "#{p.name}(#{p.health})\n"
                end

                @players.each do |player|
                    puts "\n#{player.name}'s points total:\n"
                    player.each_found_treasure do |treasure|
                        puts "#{treasure.points} total #{treasure.name} points"
                    end
                    puts "#{player.points} grand total points"
                end

                def total_points
                    @players.reduce(0) { |sum, player| sum +  player.points}
                end

                puts "\nTotal points accumulated from treasures: #{total_points}"

                puts "\n#{@title}'s High Scores:"
                scored_players = @players.sort { |a, b| b.score <=> a.score }
                scored_players.each do |player|
                    puts high_score(player)
                end
            end

        end
end
